package es.gva.pandora.service.ws.gde;

import java.io.IOException;
import java.io.StringReader;
import java.net.URISyntaxException;

import org.apache.commons.httpclient.HttpStatus;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.ParseException;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.client.utils.URIBuilder;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClientBuilder;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.apache.xerces.parsers.DOMParser;
import org.json.JSONException;
import org.json.JSONObject;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Component;
import org.springframework.web.multipart.MultipartFile;
import  org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

import es.gva.pandora.service.dto.ResourceDto;
import es.gva.pandora.service.exception.AlfrescoServerNotFoundException;
import es.gva.pandora.service.exception.AlfrescoServerResponseException;

@Component
@Qualifier("Alfresco")
public class Alfresco implements AlfrescoService {
	private static Logger logger = LoggerFactory.getLogger(Alfresco.class);
	
	@Value("${alfresco.loginUrl}")
	private String loginUrl;

	@Value("${alfresco.uploadUrl}")
	private String uploadUrl;
	
	@Value("${alfresco.downloadUrl}")
	private String downloadUrl;
	
	@Value("${alfresco.user}")
	private String user;
	
	@Value("${alfresco.password}")
	private String password;

	@Value("${alfresco.idApp}")
	private String idApp;
	
	public ResourceDto downloadFile(String gdeCode) throws AlfrescoServerNotFoundException, AlfrescoServerResponseException {
		ResourceDto resource = null;
		String ticket = GetTicket();
		
		try {
			CloseableHttpClient client = HttpClientBuilder.create().build();
			HttpGet request = new HttpGet(buildDownloadUrl(ticket, gdeCode));
		    HttpResponse response = client.execute(request);
					    
		    if (HttpStatus.SC_OK == response.getStatusLine().getStatusCode()) {
		    	resource = readDownloadResponse(response); 
	            client.close();
		    } else {
		    	client.close();
		    	logger.error("Bajada Alfresco >>> Error al descargar documento de alfresco" + response.getStatusLine().getStatusCode() + "  " + response.getStatusLine().getReasonPhrase());
		    	throw new AlfrescoServerNotFoundException("Bajada Alfresco >>> Error al descargar documento de alfresco: " + response.getStatusLine().getStatusCode() + "  " + response.getStatusLine().getReasonPhrase());
		    }
		} catch (URISyntaxException | ParseException | IOException e) {
			logger.error("Bajada Alfresco >>> Error al descargar documento de alfresco", e);
			throw new AlfrescoServerResponseException("Bajada Alfresco >>> Error al descargar documento de alfresco");
		}
		
		return resource;
	}
	
	public String uploadFile(MultipartFile file, String directorio, String titulo, String descripcion) 
							throws AlfrescoServerNotFoundException, AlfrescoServerResponseException {
		String ticket = GetTicket();
		String gde = "";
				
		try {
			CloseableHttpClient client = HttpClients.createDefault();
		    HttpPost httpPost = new HttpPost(buildUploadUrl(ticket));

		    MultipartEntityBuilder builder = MultipartEntityBuilder.create();
		    builder.addTextBody("idApp", idApp);
		    builder.addTextBody("rutaDestino", directorio);
		    builder.addTextBody("title", (titulo != null) ? titulo: "");
		    builder.addTextBody("desc", (descripcion != null) ? descripcion: " " );
		    builder.addBinaryBody("file", file.getBytes(), ContentType.DEFAULT_BINARY, titulo); 
		    
		    HttpEntity multipart = builder.build();  
		    httpPost.setEntity(multipart);
			CloseableHttpResponse response = client.execute(httpPost);
			
			if (HttpStatus.SC_OK == response.getStatusLine().getStatusCode()) {
	            gde = readUploadResponse(EntityUtils.toString(response.getEntity()));
		    } else {
		    	logger.error("Subida Alfresco >>> Respuesta fallida: " + response.getStatusLine().getStatusCode() + "  " + response.getStatusLine().getReasonPhrase());
		    	throw new AlfrescoServerResponseException("Subida Alfresco >>> Respuesta fallida: " + response.getStatusLine().getStatusCode() + "  " + response.getStatusLine().getReasonPhrase());
		    }
			client.close();
		} catch (IOException | URISyntaxException e) {
			throw new AlfrescoServerResponseException("Subida Alfresco >>> ", e);
		}
		
		return gde;
	}
		
	private String GetTicket() throws AlfrescoServerNotFoundException, AlfrescoServerResponseException {
		String ticket = "";
		
		try {			
			CloseableHttpClient client = HttpClientBuilder.create().build();
			HttpGet request = new HttpGet(buildTicketUrl());
		    HttpResponse response = client.execute(request);
					    
		    if (HttpStatus.SC_OK == response.getStatusLine().getStatusCode()) {
	            String body = EntityUtils.toString(response.getEntity());
	            ticket = readTicketResponse(body);
	            client.close();
		    } else {
		    	client.close();
		    	logger.error("Subida Alfresco >>> Error al obtener el ticket de alfresco", response.getStatusLine().getStatusCode());
		    	throw new AlfrescoServerNotFoundException("Subida Alfresco >>> Error al obtener el ticket de alfresco");
		    }
		} catch (URISyntaxException | ParseException | IOException e) {
			logger.error("Subida Alfresco >>> Error al obtener el ticket de alfresco", e);
			throw new AlfrescoServerResponseException("Subida Alfresco >>> Error al procesar la respuesta del ticket de alfresco");
		}
	    
		return ticket;
	}
	
	private String buildUploadUrl(String ticket) throws URISyntaxException {
		URIBuilder builder = new URIBuilder(uploadUrl);
		builder.addParameter("alf_ticket", ticket);
		builder.addParameter("format", "json");
		
		return builder.toString();
	}
	
	private String buildDownloadUrl(String ticket, String gde) throws URISyntaxException {
		URIBuilder builder = new URIBuilder(downloadUrl);
		builder.addParameter("alf_ticket", ticket);
		builder.addParameter("referenciaGDE", gde);
		
		return builder.toString();
	}
	
	private String buildTicketUrl() throws URISyntaxException {
		URIBuilder builder = new URIBuilder(loginUrl);
		builder.addParameter("u", user);
		builder.addParameter("pw", password);
		
		return builder.toString();
	}
	
	private ResourceDto readDownloadResponse(HttpResponse response) throws UnsupportedOperationException, IOException {
		HttpEntity entity = response.getEntity();
		MediaType type = MediaType.parseMediaType(entity.getContentType().getValue().split(";")[0]);
		ResourceDto resource = new ResourceDto();
		
		resource.setContent(EntityUtils.toByteArray(entity));
    	resource.setContentLength(entity.getContentLength());
    	resource.setContentType(type);
    	resource.setFileName(response.getFirstHeader("Content-Disposition").getElements()[0].getParameterByName("filename").getValue());
    	resource.setFileExtension(type.getSubtype());
    	
    	return resource;
	}
	
	private String readUploadResponse(String response) throws AlfrescoServerResponseException {
		try {
			JSONObject json = new JSONObject(response);
			return json.getJSONObject("upload").getString("referenciaGDE");
		} catch (JSONException e) {
			logger.error("Subida Alfresco >>> Error al procesar la respuesta de subida alfresco", e);
	    	throw new AlfrescoServerResponseException("Subida Alfresco >>> Error al procesar la respuesta de subida alfresco");
		}
	}
	
	private String readTicketResponse(String response) throws AlfrescoServerResponseException {
		DOMParser parser = new DOMParser();
		
		try {
		    parser.parse(new InputSource(new StringReader(response)));
		    Document doc = parser.getDocument();
		    return doc.getDocumentElement().getTextContent();
		} catch (SAXException | IOException e) {
			logger.error("Subida Alfresco >>> Error al procesar la respuesta del ticket de alfresco", e);
	    	throw new AlfrescoServerResponseException("Subida Alfresco >>> Error al procesar la respuesta del ticket de alfresco");
		}
	}
	
	
}
